@@ -16,6 +16,69 @@ from pytest_dbfixtures.utils import try_import, get_config
 from pytest_dbfixtures.executors import TCPExecutor
 
 
+class RabbitMqExecutor(TCPExecutor):
+
+    """
+    RabbitMQ executor to start specific rabbitmq instances.
+    """
+
+    def __init__(self, command, host, port, rabbit_ctl, environ, **kwargs):
+        """
+        Initialize RabbitMQ executor.
+
+        :param str command: rabbitmq-server location
+        :param str host: host where rabbitmq will be accessible
+        :param int port: port under which rabbitmq runs
+        :param str rabbit_ctl: rabbitctl location
+        :param dict environ: rabbitmq configuration environment variables
+        :param kwargs: see TCPExecutor for description
+        """
+        super(RabbitMqExecutor, self).__init__(command, host, port, **kwargs)
+        self.rabbit_ctl = rabbit_ctl
+        self.env = environ
+
+    def set_environ(self):
+        for env, value in self.env.iteritems():
+            os.environ[env] = value
+
+    def start(self):
+        self.set_environ()
+        TCPExecutor.start(self)
+
+    def rabbitctl_output(self, *args):
+        """
+        Queries rabbitctl with args
+
+        :param list args: list of additional args to query
+        """
+        self.set_environ()
+        ctl_command = [self.rabbit_ctl]
+        ctl_command.extend(args)
+        return subprocess.check_output(ctl_command)
+
+    def list_exchanges(self):
+        """Get exchanges defined on given rabbitmq."""
+        exchanges = []
+        output = self.rabbitctl_output('list_exchanges', 'name')
+        for exchange in output.split('\n'):
+            if exchange in ['Listing exchanges ...', '...done.']:
+                continue
+            if exchange:
+                exchanges.append(exchange)
+        return exchanges
+
+    def list_queues(self):
+        """Get queues defined on given rabbitmq."""
+        queues = []
+        output = self.rabbitctl_output('list_queues', 'name')
+        for queue in output.split('\n'):
+            if queue in ['Listing queues ...', '...done.']:
+                continue
+            if queue:
+                queues.append(queue)
+        return queues
+
+
 def rabbit_env(name):
     """
     :param str name: name of environment variable
@@ -68,7 +131,7 @@ def rabbitmq_proc(config_file=None, server=None, host=None, port=None,
         :returns pytest fixture with RabbitMQ process executor
     '''
 
-    @pytest.fixture
+    @pytest.fixture(scope='session')
     def rabbitmq_proc_fixture(request):
         """
         #. Get config.
@@ -104,61 +167,105 @@ def rabbitmq_proc(config_file=None, server=None, host=None, port=None,
         rabbit_conf['RABBITMQ_MNESIA_BASE'] = str(mnesia)
         rabbit_conf['RABBITMQ_ENABLED_PLUGINS_FILE'] = str(tmpdir / 'plugins')
 
+        environ = {}
+
         for name, value in rabbit_conf.items():
             # for new versions of rabbitmq-server
-            os.environ[name] = value
+            environ[name] = value
             # for older versions of rabbitmq-server
             prefix, name = name.split('RABBITMQ_')
-            os.environ[name] = value
+            environ[name] = value
 
         config = get_config(request)
 
+        rabbit_ctl = rabbit_ctl_file or config.rabbit.rabbit_ctl
         rabbit_server = server or config.rabbit.rabbit_server
         rabbit_host = host or config.rabbit.host
         rabbit_port = port or config.rabbit.port
 
-        os.environ['RABBITMQ_NODE_PORT'] = str(rabbit_port)
-        os.environ['NODE_PORT'] = str(rabbit_port)
+        environ['RABBITMQ_NODE_PORT'] = str(rabbit_port)
+        environ['NODE_PORT'] = str(rabbit_port)
 
-        rabbit_executor = TCPExecutor(
+        if node_name:
+            environ['RABBITMQ_NODENAME'] = node_name
+
+        rabbit_executor = RabbitMqExecutor(
             rabbit_server,
             rabbit_host,
             rabbit_port,
+            rabbit_ctl,
+            environ
         )
 
         request.addfinalizer(rabbit_executor.stop)
 
-        base_path = rabbit_path('RABBITMQ_MNESIA_BASE')
-
-        if node_name:
-            os.environ['RABBITMQ_NODENAME'] = node_name
-
-        rabbit_ctl = rabbit_ctl_file or config.rabbit.rabbit_ctl
-
         rabbit_executor.start()
 
-        # In the end we want to use `rabbitctl wait` command to be certain
-        # that our rabbitmq-server is really up and ready.
-        pid_file = base_path / rabbit_env('RABBITMQ_NODENAME') + '.pid'
-        wait_cmd = rabbit_ctl, '-q', 'wait', pid_file
-        subprocess.Popen(wait_cmd).communicate()
-
         return rabbit_executor
 
     return rabbitmq_proc_fixture
 
 
-def rabbitmq(process_fixture_name, host=None, port=None):
-    '''
-        Connects with RabbitMQ server
+def clear_rabbitmq(process, pika_connection):
+    """
+    Clear queues and exchanges from given rabbitmq process.
 
-        :param str process_fixture_name: name of RabbitMQ preocess variable
-                                         returned by rabbitmq_proc
-        :param str host: RabbitMQ server host
-        :param int port: RabbitMQ server port
+    :param RabbitMqExecutor process: rabbitmq process
+    :param pika.BlockingConnection pika_connection: connection to rabbitmq
 
-        :returns RabbitMQ connection
-    '''
+    """
+    channel = pika_connection.channel()
+    process.set_environ()
+
+    for exchange in process.list_exchanges():
+        if exchange.startswith('amq.'):
+            # ----------------------------------------------------------------
+            # From rabbit docs:
+            # https://www.rabbitmq.com/amqp-0-9-1-reference.html
+            # ----------------------------------------------------------------
+            # Exchange names starting with "amq." are reserved for pre-declared
+            # and standardised exchanges. The client MAY declare an exchange
+            # starting with "amq." if the passive option is set, or the
+            # exchange already exists. Error code: access-refused
+            # ----------------------------------------------------------------
+            continue
+        channel.exchange_delete(exchange)
+
+    for queue in process.list_queues():
+        if queue.startswith('amq.'):
+            # ----------------------------------------------------------------
+            # From rabbit docs:
+            # https://www.rabbitmq.com/amqp-0-9-1-reference.html
+            # ----------------------------------------------------------------
+            # Queue names starting with "amq." are reserved for pre-declared
+            # and standardised queues. The client MAY declare a queue starting
+            # with "amq." if the passive option is set, or the queue already
+            # exists. Error code: access-refused
+            # ----------------------------------------------------------------
+            continue
+        channel.queue_delete(queue)
+
+
+def rabbitmq(
+        process_fixture_name, host=None, port=None, teardown=clear_rabbitmq):
+    """
+    Connects with RabbitMQ server
+
+    :param str process_fixture_name: name of RabbitMQ preocess variable
+                                     returned by rabbitmq_proc
+    :param str host: RabbitMQ server host
+    :param int port: RabbitMQ server port
+    :param callable teardown: custom callable that clears rabbitmq
+
+    .. note::
+
+        calls to rabbitmqctl might be as slow or even slower
+        as restarting process. To speed up, provide Your own teardown function,
+        to remove queues and exchanges of your choosing, without querying
+        rabbitmqctl underneath.
+
+    :returns RabbitMQ connection
+    """
 
     @pytest.fixture
     def rabbitmq_factory(request):
@@ -173,7 +280,7 @@ def rabbitmq(process_fixture_name, host=None, port=None):
         """
 
         # load required process fixture
-        request.getfuncargvalue(process_fixture_name)
+        process = request.getfuncargvalue(process_fixture_name)
 
         pika, config = try_import('pika', request)
 
@@ -189,6 +296,11 @@ def rabbitmq(process_fixture_name, host=None, port=None):
         except pika.adapters.blocking_connection.exceptions.ConnectionClosed:
             print "Be sure that you're connecting rabbitmq-server >= 2.8.4"
 
+        def finalizer():
+            teardown(process, rabbit_connection)
+
+        request.addfinalizer(finalizer)
+
         return rabbit_connection
 
     return rabbitmq_factory
